@@ -14,7 +14,7 @@ use rustc_middle::ty::{error::TypeError, Predicate};
 use rustc_session::Session;
 use rustc_span::symbol::Symbol;
 use rustc_span::{FileName, Span};
-use semver::Version;
+use semver::{BuildMetadata, Prerelease, Version};
 use std::{
     cmp::Ordering,
     collections::{BTreeMap, BTreeSet, HashMap},
@@ -370,6 +370,9 @@ impl<'tcx> ChangeType<'tcx> {
     pub fn to_category(&self) -> ChangeCategory {
         // TODO: slightly messy and unreadable.
         match *self {
+            //
+            // Breaking
+            //
             ItemMadePrivate |
             KindDifference |
             StaticMutabilityChanged { now_mut: false } |
@@ -381,8 +384,7 @@ impl<'tcx> ChangeType<'tcx> {
             TypeParameterRemoved { .. } |
             VariantAdded |
             VariantRemoved |
-            VariantFieldAdded { public: true, .. } |
-            VariantFieldAdded { public: false, total_public: true, .. } |
+            VariantFieldAdded { total_public: true, .. } |
             VariantFieldRemoved { public: true, .. } |
             VariantFieldRemoved { public: false, is_enum: true, .. } |
             VariantStyleChanged { .. } |
@@ -397,12 +399,19 @@ impl<'tcx> ChangeType<'tcx> {
             TraitImplTightened |
             AssociatedItemRemoved |
             Unknown => Breaking,
+            //
+            // Technically breaking
+            //
             MethodSelfChanged { now_self: true } |
             TraitItemAdded { .. } | // either defaulted or sealed
             BoundsLoosened { trait_def: false, .. } |
             TraitImplLoosened |
             AssociatedItemAdded |
+            VariantFieldAdded { public: true, .. } |
             ItemMadePublic => TechnicallyBreaking,
+            //
+            // Non breaking
+            //
             StaticMutabilityChanged { now_mut: true } |
             VarianceLoosened |
             TypeParameterAdded { defaulted: true } |
@@ -1088,12 +1097,12 @@ impl<'tcx> ChangeSet<'tcx> {
     fn get_new_version(&self, version: &str) -> Option<String> {
         if let Ok(mut new_version) = Version::parse(version) {
             if new_version.major == 0 {
-                new_version.increment_patch();
+                increment_patch(&mut new_version);
             } else {
                 match self.max {
-                    Patch => new_version.increment_patch(),
-                    NonBreaking | TechnicallyBreaking => new_version.increment_minor(),
-                    Breaking => new_version.increment_major(),
+                    Patch => increment_patch(&mut new_version),
+                    NonBreaking | TechnicallyBreaking => increment_minor(&mut new_version),
+                    Breaking => increment_major(&mut new_version),
                 }
             }
 
@@ -1230,7 +1239,7 @@ pub mod tests {
     use std::cmp::{max, min};
 
     use rustc_span::hygiene::SyntaxContext;
-    use rustc_span::symbol::Interner;
+    use rustc_span::symbol::sym;
     use rustc_span::BytePos;
 
     /// A wrapper for `Span` that can be randomly generated.
@@ -1239,7 +1248,12 @@ pub mod tests {
 
     impl Span_ {
         pub fn inner(self) -> Span {
-            Span::new(BytePos(self.0), BytePos(self.1), SyntaxContext::root())
+            Span::new(
+                BytePos(self.0),
+                BytePos(self.1),
+                SyntaxContext::root(),
+                None,
+            )
         }
     }
 
@@ -1460,8 +1474,7 @@ pub mod tests {
         output: bool,
         changes: Vec<(ChangeType_, Option<Span_>)>,
     ) -> Change<'a> {
-        let mut interner = Interner::default();
-        let mut change = Change::new(Name::Symbol(RSymbol(interner.intern("test"))), s1, output);
+        let mut change = Change::new(Name::Symbol(RSymbol(sym::test)), s1, output);
 
         for (type_, span) in changes {
             change.insert(type_.inner(), span.map(|s| s.inner()));
@@ -1475,8 +1488,7 @@ pub mod tests {
 
     /// Construct `PathChange`s from things that can be generated.
     fn build_path_change(s1: Span, spans: Vec<(bool, Span)>) -> PathChange {
-        let mut interner = Interner::default();
-        let mut change = PathChange::new(interner.intern("test"), s1);
+        let mut change = PathChange::new(sym::test, s1);
 
         for (add, span) in spans {
             change.insert(span, add);
@@ -1546,8 +1558,7 @@ pub mod tests {
             rustc_span::create_default_session_globals_then(|| {
             let mut set = ChangeSet::default();
 
-            let mut interner = Interner::default();
-            let name = interner.intern("test");
+            let name = sym::test;
 
             let max = changes
                 .iter()
@@ -1579,8 +1590,7 @@ pub mod tests {
             rustc_span::create_default_session_globals_then(|| {
             let mut set = ChangeSet::default();
 
-            let mut interner = Interner::default();
-            let name = interner.intern("test");
+            let name = sym::test;
 
             let max = changes
                 .iter()
@@ -1614,8 +1624,7 @@ pub mod tests {
             rustc_span::create_default_session_globals_then(|| {
             let mut set = ChangeSet::default();
 
-            let mut interner = Interner::default();
-            let name = interner.intern("test");
+            let name = sym::test;
 
             let max = pchanges
                 .iter()
@@ -1700,3 +1709,24 @@ pub mod tests {
         }
     }
 }
+
+fn increment_patch(v: &mut Version) {
+    v.patch += 1;
+    v.pre = Prerelease::EMPTY;
+    v.build = BuildMetadata::EMPTY;
+}
+
+fn increment_minor(v: &mut Version) {
+    v.minor += 1;
+    v.patch = 0;
+    v.pre = Prerelease::EMPTY;
+    v.build = BuildMetadata::EMPTY;
+}
+
+fn increment_major(v: &mut Version) {
+    v.major += 1;
+    v.minor = 0;
+    v.patch = 0;
+    v.pre = Prerelease::EMPTY;
+    v.build = BuildMetadata::EMPTY;
+}
